#include "RegisterBattleClass.hpp"
#include <CreatureLib/Battling/Models/Battle.hpp>
#include <CreatureLib/Battling/Models/BattleSide.hpp>
#include "../../../../../extern/angelscript_addons/scripthandle/scripthandle.h"
#include "../../../../Battling/Battle/Battle.hpp"
#include "../../../../Battling/Pokemon/Pokemon.hpp"
#include "../../AngelScriptResolver.hpp"
#include "../../AngelScriptScript.hpp"
#include "../HelperFile.hpp"

void RegisterBattleClass::Register(asIScriptEngine* engine) {
    RegisterChoiceQueue(engine);
    RegisterBattleRandom(engine);
    RegisterBattleSide(engine);
    RegisterBattle(engine);
}

void RegisterBattleClass::RegisterChoiceQueue(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("ChoiceQueue", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("ChoiceQueue", "bool MovePokemonChoiceNext(Pokemon@ target)",
                                     asMETHOD(CreatureLib::Battling::ChoiceQueue, MoveCreatureChoiceNext),
                                     asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("ChoiceQueue", "const BaseTurnChoice@ Peek() const",
                                     asMETHOD(CreatureLib::Battling::ChoiceQueue, Peek), asCALL_THISCALL);
    Ensure(r >= 0);
}

void RegisterBattleClass::RegisterBattleRandom(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("BattleRandom", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("BattleRandom",
                                     "bool EffectChance(float chance, ExecutingMove@ move, Pokemon@ target )",
                                     asMETHOD(CreatureLib::Battling::BattleRandom, EffectChance), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("BattleRandom", "int Get()",
                                     asMETHODPR(CreatureLib::Battling::BattleRandom, Get, (), int32_t),
                                     asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("BattleRandom", "int Get(int max)",
                                     asMETHODPR(CreatureLib::Battling::BattleRandom, Get, (int32_t), int32_t),
                                     asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("BattleRandom", "int Get(int min, int max)",
                                     asMETHODPR(CreatureLib::Battling::BattleRandom, Get, (int32_t, int32_t), int32_t),
                                     asCALL_THISCALL);
    Ensure(r >= 0);
}

BORROWED_PTR_GETTER_FUNC(PkmnLib::Battling::Battle, CreatureLib::Battling::ChoiceQueue, GetCurrentTurnQueue);
CreatureLib::Battling::BattleSide* GetBattleSideWrapper(PkmnLib::Battling::Battle* battle, u8 index) {
    return battle->GetSides()[index];
}

CreatureLib::Battling::BattleParty* GetPartyWrapper(PkmnLib::Battling::Battle* battle, u8 index) {
    return battle->GetParties()[index];
}

CreatureLib::Battling::BattleParty* FindPartyWrapper(PkmnLib::Battling::Battle* battle, PkmnLib::Battling::Pokemon* p) {
    auto v = battle->FindPartyForCreature(p);
    if (v.HasValue()) {
        return v.GetValue();
    }
    return {};
}

static CScriptHandle AddVolatileWrapper(PkmnLib::Battling::Battle* obj, const ArbUt::StringView& name) {
    auto handle = CScriptHandle();
    auto* resolver = static_cast<AngelScriptResolver*>(obj->GetLibrary()->GetScriptResolver().get());
    auto script = static_cast<AngelScriptScript*>(obj->AddVolatileScript(name))->GetRawAngelscriptObject();
    handle.Set(script, resolver->GetBaseType("PkmnScript"));
    return handle;
}

static u8 GetPokemonIndexWrapper(CreatureLib::Battling::BattleSide* obj, PkmnLib::Battling::Pokemon* pokemon) {
    return obj->GetCreatureIndex(pokemon);
}

void RegisterBattleClass::RegisterBattleSide(asIScriptEngine* engine) {
    int r = engine->RegisterObjectMethod(
        "BattleSide", "bool SwapPositions(uint8 a, uint8 b)",
        asMETHODPR(CreatureLib::Battling::BattleSide, SwapPositions, (u8 a, u8 b), bool), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("BattleSide", "uint8 get_SideIndex() const property",
                                     asMETHOD(CreatureLib::Battling::BattleSide, GetSideIndex), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("BattleSide", "uint8 GetPokemonIndex(const Pokemon@ pokemon) const",
                                     asFUNCTION(GetPokemonIndexWrapper), asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
}

void RegisterBattleClass::RegisterBattle(asIScriptEngine* engine) {
    int r = engine->RegisterObjectMethod("Battle", "const BattleLibrary@ get_Library() const property",
                                         asMETHOD(PkmnLib::Battling::Battle, GetLibrary), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "bool CanUse(BaseTurnChoice@ choice)",
                                     asMETHOD(PkmnLib::Battling::Battle, CanUse), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "bool get_CanFlee() const property",
                                     asMETHOD(PkmnLib::Battling::Battle, CanFlee), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "BattleRandom@ get_Random() const property",
                                     asMETHOD(PkmnLib::Battling::Battle, GetRandom), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "ChoiceQueue@ get_TurnQueue() const property",
                                     asFUNCTION(GetCurrentTurnQueueWrapper), asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "ref@ AddVolatile(const constString &in name)",
                                     asFUNCTION(AddVolatileWrapper), asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod(
        "Battle", "void RemoveVolatile(const constString &in name) const",
        asMETHODPR(PkmnLib::Battling::Battle, RemoveVolatileScript, (const ArbUt::BasicStringView&), void),
        asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod(
        "Battle", "void SetWeather(const constString &in name) const",
        asMETHODPR(PkmnLib::Battling::Battle, SetWeather, (const ArbUt::StringView&), void), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "void ClearWeather(const constString &in name) const",
                                     asMETHODPR(PkmnLib::Battling::Battle, ClearWeather, (), void), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod(
        "Battle", "const constString& GetWeatherName() const",
        asMETHODPR(PkmnLib::Battling::Battle, GetWeatherName, (), const ArbUt::StringView&), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "BattleSide@ GetBattleSide(uint8 index)",
                                     asFUNCTION(GetBattleSideWrapper), asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "BattleParty@ GetParty(uint8 index)", asFUNCTION(GetPartyWrapper),
                                     asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("Battle", "BattleParty@ FindPartyForPokemon(Pokemon@ pokemon)",
                                     asFUNCTION(FindPartyWrapper), asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
}
