#include "RegisterTurnChoices.hpp"
#include <CreatureLib/Battling/TurnChoices/AttackTurnChoice.hpp>
#include <CreatureLib/Battling/TurnChoices/BaseTurnChoice.hpp>
#include <CreatureLib/Battling/TurnChoices/SwitchTurnChoice.hpp>
#include "../HelperFile.hpp"
#include "../RefCast.hpp"

BORROWED_PTR_GETTER_FUNC(CreatureLib::Battling::BaseTurnChoice, CreatureLib::Battling::Creature, GetUser);

void RegisterTurnChoices::Register(asIScriptEngine* engine) {
    RegisterTurnChoiceKindEnum(engine);
    RegisterBaseTurnChoice(engine);
    RegisterMoveTurnChoice(engine);
    RegisterSwitchTurnChoice(engine);
    RegisterFleeTurnChoice(engine);
}
void RegisterTurnChoices::RegisterTurnChoiceKindEnum(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterEnum("TurnChoiceKind");
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Pass", (int)CreatureLib::Battling::TurnChoiceKind::Pass);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Attack", (int)CreatureLib::Battling::TurnChoiceKind::Attack);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Item", (int)CreatureLib::Battling::TurnChoiceKind::Item);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Switch", (int)CreatureLib::Battling::TurnChoiceKind::Switch);
    Ensure(r >= 0);
    r = engine->RegisterEnumValue("TurnChoiceKind", "Flee", (int)CreatureLib::Battling::TurnChoiceKind::Flee);
    Ensure(r >= 0);
}
void RegisterTurnChoices::RegisterBaseTurnChoice(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("BaseTurnChoice", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("BaseTurnChoice", "TurnChoiceKind get_Kind() const property",
                                     asMETHOD(CreatureLib::Battling::BaseTurnChoice, GetKind), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("BaseTurnChoice", "const Pokemon@ get_User() const property",
                                     asFUNCTION(GetUserWrapper), asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
}

void RegisterTurnChoices::RegisterMoveTurnChoice(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("MoveTurnChoice", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("MoveTurnChoice", "TurnChoiceKind get_Kind() const property",
                                     asMETHOD(CreatureLib::Battling::AttackTurnChoice, GetKind), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("MoveTurnChoice", "Pokemon@ get_User() const property", asFUNCTION(GetUserWrapper),
                                     asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("MoveTurnChoice", "LearnedMove@ get_Move() const property",
                                     asMETHOD(CreatureLib::Battling::AttackTurnChoice, GetAttack), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("MoveTurnChoice", "int8 get_Priority() const property",
                                     asMETHOD(CreatureLib::Battling::AttackTurnChoice, GetPriority), asCALL_THISCALL);
    Ensure(r >= 0);

    r = engine->RegisterObjectMethod(
        "BaseTurnChoice", "MoveTurnChoice@ opCast()",
        asFUNCTION((refCast<CreatureLib::Battling::BaseTurnChoice, CreatureLib::Battling::AttackTurnChoice>)),
        asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod(
        "MoveTurnChoice", "BaseTurnChoice@ opImplCast()",
        asFUNCTION((refCast<CreatureLib::Battling::AttackTurnChoice, CreatureLib::Battling::BaseTurnChoice>)),
        asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
}

void RegisterTurnChoices::RegisterSwitchTurnChoice(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("SwitchTurnChoice", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("SwitchTurnChoice", "TurnChoiceKind get_Kind() const property",
                                     asMETHOD(CreatureLib::Battling::SwitchTurnChoice, GetKind), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("SwitchTurnChoice", "const Pokemon@ get_User() const property",
                                     asFUNCTION(GetUserWrapper), asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("SwitchTurnChoice", "Pokemon@ get_NewPokemon() const property",
                                     asMETHOD(CreatureLib::Battling::SwitchTurnChoice, GetNewCreature),
                                     asCALL_THISCALL);
    Ensure(r >= 0);

    r = engine->RegisterObjectMethod(
        "BaseTurnChoice", "SwitchTurnChoice@ opCast()",
        asFUNCTION((refCast<CreatureLib::Battling::BaseTurnChoice, CreatureLib::Battling::SwitchTurnChoice>)),
        asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod(
        "SwitchTurnChoice", "BaseTurnChoice@ opImplCast()",
        asFUNCTION((refCast<CreatureLib::Battling::AttackTurnChoice, CreatureLib::Battling::BaseTurnChoice>)),
        asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
}
void RegisterTurnChoices::RegisterFleeTurnChoice(asIScriptEngine* engine) {
    [[maybe_unused]] int r = engine->RegisterObjectType("FleeTurnChoice", 0, asOBJ_REF | asOBJ_NOCOUNT);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("FleeTurnChoice", "TurnChoiceKind get_Kind() const property",
                                     asMETHOD(CreatureLib::Battling::FleeTurnChoice, GetKind), asCALL_THISCALL);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod("FleeTurnChoice", "const Pokemon@ get_User() const property",
                                     asFUNCTION(GetUserWrapper), asCALL_CDECL_OBJFIRST);
    Ensure(r >= 0);

    r = engine->RegisterObjectMethod(
        "BaseTurnChoice", "FleeTurnChoice@ opCast()",
        asFUNCTION((refCast<CreatureLib::Battling::BaseTurnChoice, CreatureLib::Battling::FleeTurnChoice>)),
        asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
    r = engine->RegisterObjectMethod(
        "FleeTurnChoice", "BaseTurnChoice@ opImplCast()",
        asFUNCTION((refCast<CreatureLib::Battling::FleeTurnChoice, CreatureLib::Battling::BaseTurnChoice>)),
        asCALL_CDECL_OBJLAST);
    Ensure(r >= 0);
}
